import React, { Component } from 'react';
import {
  Animated,
  Platform,
  StatusBar,
  StyleSheet,
  Text,
  View,
  RefreshControl,
} from 'react-native';

const DEFAULT_HEADER_MAX_HEIGHT = 300;
const DEFAULT_HEADER_MIN_HEIGHT = Platform.OS === 'ios' ? 60 : 73;
const DEFAULT_HEADER_SCROLL_DISTANCE = 240;

export default class RNParallaxScrollView extends Component {
  constructor(props) {
    super(props);
    this.state = {
      scrollY: new Animated.Value(
        // iOS has negative initial scroll value because content inset...
        Platform.OS === 'ios' ? - (this.props.headerMaxHeight ? this.props.headerMaxHeight : DEFAULT_HEADER_MAX_HEIGHT) : 0,
      ),
      refreshing: false,
    };
  }

  render() {
    // Because of content inset the scroll value will be negative on iOS so bring
    // it back to 0.
    const {
      headerMaxHeight,
      headerMinHeight,
      headerScrollDistance,
      headerBarStyles,
      headerWrapperStyles,
      transformHeader,
      backgroundImageStyles
    } = this.props;
    const HEADER_MAX_HEIGHT = headerMaxHeight ? headerMaxHeight : DEFAULT_HEADER_MAX_HEIGHT;
    const HEADER_MIN_HEIGHT = headerMinHeight ? headerMinHeight : DEFAULT_HEADER_MIN_HEIGHT;
    const HEADER_SCROLL_DISTANCE = headerScrollDistance ? headerScrollDistance : DEFAULT_HEADER_SCROLL_DISTANCE;

    const scrollY = Animated.add(
      this.state.scrollY,
      Platform.OS === 'ios' ? HEADER_MAX_HEIGHT : 0,
    );
    const headerTranslate = scrollY.interpolate({
      inputRange: [0, HEADER_SCROLL_DISTANCE],
      outputRange: [0, -HEADER_SCROLL_DISTANCE],
      extrapolate: 'clamp',
    });

    const imageOpacity = scrollY.interpolate({
      inputRange: [0, HEADER_SCROLL_DISTANCE / 2, HEADER_SCROLL_DISTANCE],
      outputRange: [1, 1, 0],
      extrapolate: 'clamp',
    });
    const imageTranslate = scrollY.interpolate({
      inputRange: [0, HEADER_SCROLL_DISTANCE],
      outputRange: [0, 100],
      extrapolate: 'clamp',
    });

    const titleScale = scrollY.interpolate({
      inputRange: [0, HEADER_SCROLL_DISTANCE / 2, HEADER_SCROLL_DISTANCE],
      outputRange: [1, 1, 0.8],
      extrapolate: 'clamp',
    });
    const titleTranslate = scrollY.interpolate({
      inputRange: [0, HEADER_SCROLL_DISTANCE / 2, HEADER_SCROLL_DISTANCE],
      outputRange: [0, 0, -8],
      extrapolate: 'clamp',
    });

    return (
      <View style={styles.fill}>
        <StatusBar
          translucent
          barStyle="light-content"
          backgroundColor="rgba(0, 0, 0, 0.251)"
        />
        <Animated.ScrollView
          style={styles.fill}
          scrollEventThrottle={1}
          onScroll={Animated.event(
            [{ nativeEvent: { contentOffset: { y: this.state.scrollY } } }],
            { useNativeDriver: true },
          )}
          refreshControl={
            <RefreshControl
              refreshing={this.state.refreshing}
              onRefresh={() => {
                this.setState({ refreshing: true });
                setTimeout(() => this.setState({ refreshing: false }), 1000);
              }}
              // Android offset for RefreshControl
              progressViewOffset={HEADER_MAX_HEIGHT}
            />
          }
          // iOS offset for RefreshControl
          contentInset={{
            top: HEADER_MAX_HEIGHT,
          }}
          contentOffset={{
            y: -HEADER_MAX_HEIGHT,
          }}
        >
          {this.props.renderScrollViewContent()}
        </Animated.ScrollView>
        <Animated.View
          pointerEvents="none"
          style={[
            styles.header,
            { transform: [{ translateY: headerTranslate }] },
            {height: HEADER_MAX_HEIGHT},
            headerWrapperStyles && headerWrapperStyles
          ]}
        >
          <Animated.Image
            style={[
              styles.backgroundImage,
              backgroundImageStyles && backgroundImageStyles,
              {
                opacity: imageOpacity,
                transform: [{ translateY: imageTranslate }],
              },
              { height: HEADER_MAX_HEIGHT }
            ]}
            source={{uri:this.props.thumbnail}}
          />
        </Animated.View>
        <Animated.View
          style={[
            styles.bar,
            transformHeader &&
            {
              transform: [
                { scale: titleScale },
                { translateY: titleTranslate },
              ],
            },
            headerBarStyles && headerBarStyles
          ]}
        >
        {this.props.renderFixedHeader()}
        </Animated.View>
      </View>
    );
  }
}

const styles = StyleSheet.create({
  fill: {
    flex: 1,
  },
  content: {
    flex: 1,
  },
  header: {
    position: 'absolute',
    top: 0,
    left: 0,
    right: 0,
    backgroundColor: '#03A9F4',
    overflow: 'hidden',
  },
  backgroundImage: {
    position: 'absolute',
    top: 0,
    left: 0,
    right: 0,
    width: null,
    resizeMode: 'cover',
  },
  bar: {
    backgroundColor: 'transparent',
    marginTop: Platform.OS === 'ios' ? 28 : 38,
    position: 'absolute',
    top: 0,
    left: 0,
    right: 0,
    justifyContent:'center',
    alignItems:'center'
  },
  row: {
    height: 40,
    margin: 16,
    backgroundColor: '#D3D3D3',
    alignItems: 'center',
    justifyContent: 'center',
  },
});

module.exports = RNParallaxScrollView;